@@ -575,7 +575,7 @@ public abstract class AbstractFile extends AbstractContent {
 		long imgOffset = -1;
 		for (TskFileRange byteRange : getRanges()) {
 
-			// if fileOffset is within the current byteRange, calcuate the image
+			// if fileOffset is within the current byteRange, calculate the image
 			// offset and break
 			long rangeLength = byteRange.getByteLen();
 			if (fileOffset < rangeLength) {
@@ -591,7 +591,8 @@ public abstract class AbstractFile extends AbstractContent {
 	}
 
 	/**
-	 * is this a virtual file or directory
+	 * is this a virtual file or directory that was created by The Sleuth 
+     * Kit or Autopsy for general structure and organization. 
 	 *
 	 * @return true if it's virtual, false otherwise
 	 */
@@ -602,17 +603,22 @@ public abstract class AbstractFile extends AbstractContent {
 	}
 
 	/**
-	 * Is this object a file
+	 * Is this object a file.  Should return true for all types of files, 
+     * including file system, logical, derived, layout, and slack space 
+     * for files.
 	 *
 	 * @return true if a file, false otherwise
 	 */
 	public boolean isFile() {
-		return metaType.equals(TSK_FS_META_TYPE_ENUM.TSK_FS_META_TYPE_REG);
+		return metaType.equals(TSK_FS_META_TYPE_ENUM.TSK_FS_META_TYPE_REG) 
+				||(metaType.equals(TSK_FS_META_TYPE_ENUM.TSK_FS_META_TYPE_UNDEF) 
+				&& dirType.equals(TSK_FS_NAME_TYPE_ENUM.REG));
 
 	}
 
 	/**
-	 * Is this object a directory
+	 * Is this object a directory.  Should return true for file system
+     * folders and virtual folders. 
 	 *
 	 * @return true if directory, false otherwise
 	 */
@@ -1187,7 +1193,7 @@ public abstract class AbstractFile extends AbstractContent {
 		/*
 		 * NOTE: previously attrId used to be stored in AbstractFile as (signed)
 		 * short even though it is stored as uint16 in TSK. In extremely rare
-		 * occurances attrId can be larger than what a signed short can hold
+		 * occurrences attrId can be larger than what a signed short can hold
 		 * (2^15). Changes were made to AbstractFile to store attrId as integer.
 		 * Therefore this method has been deprecated. For backwards
 		 * compatibility, attribute ids that are larger than 32K are converted
