@@ -1,7 +1,7 @@
 /*
  * Sleuth Kit Data Model
  *
- * Copyright 2011-2016 Basis Technology Corp.
+ * Copyright 2011-2017 Basis Technology Corp.
  * Contact: carrier <at> sleuthkit <dot> org
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -24,10 +24,13 @@ import java.io.IOException;
 import java.text.DateFormat;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.GregorianCalendar;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.TimeZone;
 import java.util.UUID;
 import org.sleuthkit.datamodel.TskData.TSK_FS_ATTR_TYPE_ENUM;
@@ -49,12 +52,6 @@ public class SleuthkitJNI {
 		LibraryUtils.loadSleuthkitJNI();
 	}
 
-	/*
-	 * A monitor used to guard access to cached SleuthKit image and file system
-	 * handles.
-	 */
-	private static final Object cacheLock = new Object();
-
 	/**
 	 * Constructor for the utility class that provides a interface to the
 	 * SleuthKit via JNI.
@@ -63,15 +60,17 @@ public class SleuthkitJNI {
 	}
 
 	/**
-	 * Encapsulates a handle to a SleuthKit case database with support for
-	 * adding images to the database.
+	 * Cache of all handles allocated in the JNI layer. Used for: (a) quick
+	 * lookup of frequently used handles (e.g. file system and image) (b)
+	 * ensuring all handles passed in by clients of SleuthkitJNI are valid. (c)
+	 * consistent cleanup of handles on closure.
 	 */
-	public static class CaseDbHandle {
+	private static class HandleCache {
 
 		/*
-		 * A pointer to a TskCaseDb object.
+		 * A monitor used to guard access to cached Sleuthkit JNI handles.
 		 */
-		private final long caseDbPointer;
+		private static final Object cacheLock = new Object();
 
 		/*
 		 * A SleuthKit image handle cache implemented as a mappng of
@@ -85,6 +84,92 @@ public class SleuthkitJNI {
 		 */
 		private static final Map<Long, Map<Long, Long>> fsHandleCache = new HashMap<Long, Map<Long, Long>>();
 
+		/*
+		 * The collection of open file handles. We will only allow requests
+		 * through to the C code if the file handle exists in this collection.
+		 */
+		private static final Set<Long> fileHandleCache = new HashSet<Long>();
+
+		private static final Map<Long, List<Long>> fileSystemToFileHandles = new HashMap<Long, List<Long>>();
+
+		private static final String INVALID_FILE_HANDLE = "Invalid file handle."; //NON-NLS
+
+		/**
+		 * Add a new file handle to the cache.
+		 *
+		 * @param fileHandle The new file handle.
+		 * @param fsHandle   The file system handle in which the file lives.
+		 */
+		private static void addFileHandle(long fileHandle, long fsHandle) {
+			synchronized (cacheLock) {
+				// Add to collection of open file handles.
+				fileHandleCache.add(fileHandle);
+
+				// Add to map of file system to file handles.
+				if (fileSystemToFileHandles.containsKey(fsHandle)) {
+					fileSystemToFileHandles.get(fsHandle).add(fileHandle);
+				} else {
+					fileSystemToFileHandles.put(fsHandle, new ArrayList<Long>(Arrays.asList(fileHandle)));
+				}
+			}
+		}
+
+		private static void removeFileHandle(long fileHandle) {
+			synchronized (cacheLock) {
+				// Remove from collection of open file handles.
+				fileHandleCache.remove(fileHandle);
+			}
+		}
+
+		private static boolean isValidFileHandle(long fileHandle) {
+			synchronized (cacheLock) {
+				return fileHandleCache.contains(fileHandle);
+			}
+		}
+
+		private static void closeHandlesAndClearCache() throws TskCoreException {
+			synchronized (cacheLock) {
+				/*
+				 * Close any cached file system handles.
+				 */
+				for (Map<Long, Long> imageToFsMap : fsHandleCache.values()) {
+					for (Long fsHandle : imageToFsMap.values()) {
+						// First close all open file handles for the file system.
+						for (Long fileHandle : fileSystemToFileHandles.get(fsHandle)) {
+							closeFile(fileHandle);
+						}
+						// Then close the file system handle.
+						closeFsNat(fsHandle);
+					}
+				}
+
+				/*
+				 * Close any cached image handles.
+				 */
+				for (Long imageHandle : imageHandleCache.values()) {
+					closeImgNat(imageHandle);
+				}
+
+				fsHandleCache.clear();
+				imageHandleCache.clear();
+				fileHandleCache.clear();
+				fileSystemToFileHandles.clear();
+			}
+
+		}
+	}
+
+	/**
+	 * Encapsulates a handle to a SleuthKit case database with support for
+	 * adding images to the database.
+	 */
+	public static class CaseDbHandle {
+
+		/*
+		 * A pointer to a TskCaseDb object.
+		 */
+		private final long caseDbPointer;
+
 		/**
 		 * Constructs an object that encapsulates a handle to a SleuthKit case
 		 * database with support for adding images to the database.
@@ -102,27 +187,7 @@ public class SleuthkitJNI {
 		 *                          operation.
 		 */
 		void free() throws TskCoreException {
-			synchronized (cacheLock) {
-				/*
-				 * Close any cached file system handles.
-				 */
-				for (Map<Long, Long> imageToFsMap : fsHandleCache.values()) {
-					for (Long fsHandle : imageToFsMap.values()) {
-						closeFsNat(fsHandle);
-					}
-				}
-
-				/*
-				 * Close any cached image handles.
-				 */
-				for (Long imageHandle : imageHandleCache.values()) {
-					closeImgNat(imageHandle);
-				}
-
-				fsHandleCache.clear();
-				imageHandleCache.clear();
-			}
-
+			HandleCache.closeHandlesAndClearCache();
 			SleuthkitJNI.closeCaseDbNat(caseDbPointer);
 		}
 
@@ -152,7 +217,7 @@ public class SleuthkitJNI {
 		long addImageInfo(long deviceObjId, List<String> imageFilePaths, String timeZone) throws TskCoreException {
 			try {
 				long tskAutoDbPointer = initializeAddImgNat(caseDbPointer, timezoneLongToShort(timeZone), false, false, false);
-				runAddImgNat(tskAutoDbPointer, UUID.randomUUID().toString(), imageFilePaths.toArray(new String[0]), imageFilePaths.size(), timeZone);
+				runOpenAndAddImgNat(tskAutoDbPointer, UUID.randomUUID().toString(), imageFilePaths.toArray(new String[0]), imageFilePaths.size(), timeZone);
 				return commitAddImgNat(tskAutoDbPointer);
 			} catch (TskDataException ex) {
 				throw new TskCoreException("Error adding image to case database", ex);
@@ -168,12 +233,15 @@ public class SleuthkitJNI {
 		 *                         unallocated space.
 		 * @param skipFatFsOrphans Pass true to skip processing of orphan files
 		 *                         for FAT file systems.
+		 * @param imageCopyPath    Path to which a copy of the image should be
+		 *                         written. Use the empty string to disable
+		 *                         image writing.
 		 *
 		 * @return An object that can be used to exercise fine-grained control
 		 *         of the process of adding the image to the case database.
 		 */
-		AddImageProcess initAddImageProcess(String timeZone, boolean addUnallocSpace, boolean skipFatFsOrphans) {
-			return new AddImageProcess(timeZone, addUnallocSpace, skipFatFsOrphans);
+		AddImageProcess initAddImageProcess(String timeZone, boolean addUnallocSpace, boolean skipFatFsOrphans, String imageCopyPath) {
+			return new AddImageProcess(timeZone, addUnallocSpace, skipFatFsOrphans, imageCopyPath);
 		}
 
 		/**
@@ -185,7 +253,9 @@ public class SleuthkitJNI {
 			private final String timeZone;
 			private final boolean addUnallocSpace;
 			private final boolean skipFatFsOrphans;
+			private final String imageWriterPath;
 			private volatile long tskAutoDbPointer;
+			private boolean isCanceled;
 
 			/**
 			 * Constructs an object that encapsulates a multi-step process to
@@ -196,12 +266,17 @@ public class SleuthkitJNI {
 			 *                         unallocated space.
 			 * @param skipFatFsOrphans Pass true to skip processing of orphan
 			 *                         files for FAT file systems.
+			 * @param imageWriterPath  Path that a copy of the image should be
+			 *                         written to. Use empty string to disable
+			 *                         image writing
 			 */
-			private AddImageProcess(String timeZone, boolean addUnallocSpace, boolean skipFatFsOrphans) {
+			private AddImageProcess(String timeZone, boolean addUnallocSpace, boolean skipFatFsOrphans, String imageWriterPath) {
 				this.timeZone = timeZone;
 				this.addUnallocSpace = addUnallocSpace;
 				this.skipFatFsOrphans = skipFatFsOrphans;
+				this.imageWriterPath = imageWriterPath;
 				tskAutoDbPointer = 0;
+				this.isCanceled = false;
 			}
 
 			/**
@@ -222,17 +297,22 @@ public class SleuthkitJNI {
 			 *                          the process)
 			 */
 			public void run(String deviceId, String[] imageFilePaths) throws TskCoreException, TskDataException {
-				if (0 != tskAutoDbPointer) {
-					throw new TskCoreException("Add image process already started");
-				}
-
+				long imageHandle = 0;
 				synchronized (this) {
-					tskAutoDbPointer = initAddImgNat(caseDbPointer, timezoneLongToShort(timeZone), addUnallocSpace, skipFatFsOrphans);
+					if (0 != tskAutoDbPointer) {
+						throw new TskCoreException("Add image process already started");
+					}
+					if (!isCanceled) { //with isCanceled being guarded by this it will have the same value everywhere in this synchronized block
+						imageHandle = openImage(imageFilePaths, false);
+						tskAutoDbPointer = initAddImgNat(caseDbPointer, timezoneLongToShort(timeZone), addUnallocSpace, skipFatFsOrphans);
+					}
+					if (0 == tskAutoDbPointer) {
+						throw new TskCoreException("initAddImgNat returned a NULL TskAutoDb pointer");
+					}
 				}
-				if (0 == tskAutoDbPointer) {
-					throw new TskCoreException("initAddImgNat returned a NULL TskAutoDb pointer");
+				if (imageHandle != 0) {
+					runAddImgNat(tskAutoDbPointer, deviceId, imageHandle, timeZone, imageWriterPath);
 				}
-				runAddImgNat(tskAutoDbPointer, deviceId, imageFilePaths, imageFilePaths.length, timeZone);
 			}
 
 			/**
@@ -244,12 +324,11 @@ public class SleuthkitJNI {
 			 * @throws TskCoreException if a critical error occurs within the
 			 *                          SleuthKit.
 			 */
-			public void stop() throws TskCoreException {
-				if (tskAutoDbPointer == 0) {
-					throw new TskCoreException("AddImgProcess::stop: AutoDB pointer is NULL");
+			public synchronized void stop() throws TskCoreException {
+				isCanceled = true;
+				if (tskAutoDbPointer != 0) {
+					stopAddImgNat(tskAutoDbPointer);
 				}
-
-				stopAddImgNat(tskAutoDbPointer);
 			}
 
 			/**
@@ -296,7 +375,7 @@ public class SleuthkitJNI {
 			 * @return The directory
 			 */
 			public synchronized String currentDirectory() {
-				return tskAutoDbPointer == 0 ? "NO_INFO" : getCurDirNat(tskAutoDbPointer); //NON-NLS
+				return tskAutoDbPointer == 0 ? "" : getCurDirNat(tskAutoDbPointer); //NON-NLS
 			}
 
 			/**
@@ -411,6 +490,26 @@ public class SleuthkitJNI {
 	 *                          TSK
 	 */
 	public static long openImage(String[] imageFiles) throws TskCoreException {
+		return openImage(imageFiles, true);
+	}
+
+	/**
+	 * open the image and return the image info pointer This is a temporary
+	 * measure to allow ingest of multiple local disks on the same drive letter.
+	 * We need to clear the cache to make sure cached data from the first drive
+	 * is not used.
+	 *
+	 * @param imageFiles the paths to the images
+	 * @param useCache   true if the image handle cache should be used, false to
+	 *                   always go to TSK to open a fresh copy
+	 *
+	 * @return the image info pointer
+	 *
+	 * @throws TskCoreException exception thrown if critical error occurs within
+	 *                          TSK
+	 */
+	private static long openImage(String[] imageFiles, boolean useCache) throws TskCoreException {
+
 		long imageHandle;
 
 		StringBuilder keyBuilder = new StringBuilder();
@@ -419,15 +518,22 @@ public class SleuthkitJNI {
 		}
 		final String imageKey = keyBuilder.toString();
 
-		synchronized (cacheLock) {
-			if (CaseDbHandle.imageHandleCache.containsKey(imageKey)) //get from cache
+		synchronized (HandleCache.cacheLock) {
+			// If we're getting a fresh copy, remove any existing cache references
+			if (!useCache && HandleCache.imageHandleCache.containsKey(imageKey)) {
+				long tempImageHandle = HandleCache.imageHandleCache.get(imageKey);
+				HandleCache.fsHandleCache.remove(tempImageHandle);
+				HandleCache.imageHandleCache.remove(imageKey);
+			}
+
+			if (useCache && HandleCache.imageHandleCache.containsKey(imageKey)) //get from cache
 			{
-				imageHandle = CaseDbHandle.imageHandleCache.get(imageKey);
+				imageHandle = HandleCache.imageHandleCache.get(imageKey);
 			} else {
 				//open new handle and cache it
 				imageHandle = openImgNat(imageFiles, imageFiles.length);
-				CaseDbHandle.fsHandleCache.put(imageHandle, new HashMap<Long, Long>());
-				CaseDbHandle.imageHandleCache.put(imageKey, imageHandle);
+				HandleCache.fsHandleCache.put(imageHandle, new HashMap<Long, Long>());
+				HandleCache.imageHandleCache.put(imageKey, imageHandle);
 			}
 		}
 		return imageHandle;
@@ -480,8 +586,8 @@ public class SleuthkitJNI {
 	 */
 	public static long openFs(long imgHandle, long fsOffset) throws TskCoreException {
 		long fsHandle;
-		synchronized (cacheLock) {
-			final Map<Long, Long> imgOffSetToFsHandle = CaseDbHandle.fsHandleCache.get(imgHandle);
+		synchronized (HandleCache.cacheLock) {
+			final Map<Long, Long> imgOffSetToFsHandle = HandleCache.fsHandleCache.get(imgHandle);
 			if (imgOffSetToFsHandle.containsKey(fsOffset)) {
 				//return cached
 				fsHandle = imgOffSetToFsHandle.get(fsOffset);
@@ -511,7 +617,7 @@ public class SleuthkitJNI {
 		/*
 		 * NOTE: previously attrId used to be stored in AbstractFile as (signed)
 		 * short even though it is stored as uint16 in TSK. In extremely rare
-		 * occurances attrId can be larger than what a signed short can hold
+		 * occurrences attrId can be larger than what a signed short can hold
 		 * (2^15). Changes were made to AbstractFile to store attrId as integer.
 		 * However, a depricated method still exists in AbstractFile to get
 		 * attrId as short. In that method we convert attribute ids that are
@@ -519,7 +625,9 @@ public class SleuthkitJNI {
 		 * need to convert negative attribute id to uint16 which is what TSK is
 		 * using to store attribute id.
 		 */
-		return openFileNat(fsHandle, fileId, attrType.getValue(), convertSignedToUnsigned(attrId));
+		long fileHandle = openFileNat(fsHandle, fileId, attrType.getValue(), convertSignedToUnsigned(attrId));
+		HandleCache.addFileHandle(fileHandle, fsHandle);
+		return fileHandle;
 	}
 
 	/**
@@ -614,24 +722,24 @@ public class SleuthkitJNI {
 	}
 
 	/**
-	 * enum used to tell readFileNat whether the offset is from the beginning
-	 * of the file or from the beginning of the slack space.
+	 * enum used to tell readFileNat whether the offset is from the beginning of
+	 * the file or from the beginning of the slack space.
 	 */
-	private enum TSK_FS_FILE_READ_OFFSET_TYPE_ENUM{
+	private enum TSK_FS_FILE_READ_OFFSET_TYPE_ENUM {
 		START_OF_FILE(0),
 		START_OF_SLACK(1);
-		
+
 		private final int val;
-		
-		TSK_FS_FILE_READ_OFFSET_TYPE_ENUM(int val){
+
+		TSK_FS_FILE_READ_OFFSET_TYPE_ENUM(int val) {
 			this.val = val;
 		}
-		
-		int getValue(){
+
+		int getValue() {
 			return val;
 		}
 	}
-	
+
 	/**
 	 * reads data from an file
 	 *
@@ -647,9 +755,13 @@ public class SleuthkitJNI {
 	 *                          TSK
 	 */
 	public static int readFile(long fileHandle, byte[] readBuffer, long offset, long len) throws TskCoreException {
+		if (!HandleCache.isValidFileHandle(fileHandle)) {
+			throw new TskCoreException(HandleCache.INVALID_FILE_HANDLE);
+		}
+
 		return readFileNat(fileHandle, readBuffer, offset, TSK_FS_FILE_READ_OFFSET_TYPE_ENUM.START_OF_FILE.getValue(), len);
 	}
-	
+
 	/**
 	 * reads data from the slack space of a file
 	 *
@@ -665,8 +777,12 @@ public class SleuthkitJNI {
 	 *                          TSK
 	 */
 	public static int readFileSlack(long fileHandle, byte[] readBuffer, long offset, long len) throws TskCoreException {
+		if (!HandleCache.isValidFileHandle(fileHandle)) {
+			throw new TskCoreException(HandleCache.INVALID_FILE_HANDLE);
+		}
+
 		return readFileNat(fileHandle, readBuffer, offset, TSK_FS_FILE_READ_OFFSET_TYPE_ENUM.START_OF_SLACK.getValue(), len);
-	}	
+	}
 
 	/**
 	 * Get human readable (some what) details about a file. This is the same as
@@ -679,6 +795,10 @@ public class SleuthkitJNI {
 	 * @throws TskCoreException if errors occurred
 	 */
 	public static List<String> getFileMetaDataText(long fileHandle) throws TskCoreException {
+		if (!HandleCache.isValidFileHandle(fileHandle)) {
+			throw new TskCoreException(HandleCache.INVALID_FILE_HANDLE);
+		}
+
 		try {
 			java.io.File tmp = java.io.File.createTempFile("tsk", ".txt");
 
@@ -744,7 +864,12 @@ public class SleuthkitJNI {
 	 * @param fileHandle pointer to file structure in sleuthkit
 	 */
 	public static void closeFile(long fileHandle) {
+		if (!HandleCache.isValidFileHandle(fileHandle)) {
+			// File handle is not open so this is a no-op.
+			return;
+		}
 		closeFileNat(fileHandle);
+		HandleCache.removeFileHandle(fileHandle);
 	}
 
 	/**
@@ -966,6 +1091,38 @@ public class SleuthkitJNI {
 		return timezoneShortForm;
 	}
 
+	/**
+	 * Fills in any gaps in the image created by image writer.
+	 *
+	 * @param imgHandle
+	 *
+	 * @throws TskCoreException exception thrown if critical error occurs within
+	 *                          TSK
+	 */
+	public static int finishImageWriter(long imgHandle) throws TskCoreException {
+		return finishImageWriterNat(imgHandle);
+	}
+
+	/**
+	 * Get the current progress of the finish image process (0-100)
+	 *
+	 * @param imgHandle
+	 *
+	 * @return Percentage of blocks completed (0-100)
+	 */
+	public static int getFinishImageProgress(long imgHandle) {
+		return getFinishImageProgressNat(imgHandle);
+	}
+
+	/**
+	 * Cancel the finish image process
+	 *
+	 * @param imgHandle
+	 */
+	public static void cancelFinishImage(long imgHandle) {
+		cancelFinishImageNat(imgHandle);
+	}
+
 	/**
 	 * Get size of a device (physical, logical device, image) pointed to by
 	 * devPath
@@ -980,8 +1137,8 @@ public class SleuthkitJNI {
 	public static long findDeviceSize(String devPath) throws TskCoreException {
 		return findDeviceSizeNat(devPath);
 	}
-	
-	public static boolean isImageSupported(String imagePath){
+
+	public static boolean isImageSupported(String imagePath) {
 		return isImageSupportedNat(imagePath);
 	}
 
@@ -1039,7 +1196,9 @@ public class SleuthkitJNI {
 
 	private static native long initializeAddImgNat(long db, String timezone, boolean addFileSystems, boolean addUnallocSpace, boolean skipFatFsOrphans) throws TskCoreException;
 
-	private static native void runAddImgNat(long process, String deviceId, String[] imgPath, int splits, String timezone) throws TskCoreException, TskDataException;
+	private static native void runOpenAndAddImgNat(long process, String deviceId, String[] imgPath, int splits, String timezone) throws TskCoreException, TskDataException;
+
+	private static native void runAddImgNat(long process, String deviceId, long a_img_info, String timeZone, String imageWriterPath) throws TskCoreException, TskDataException;
 
 	private static native void stopAddImgNat(long process) throws TskCoreException;
 
@@ -1080,7 +1239,13 @@ public class SleuthkitJNI {
 	private static native long findDeviceSizeNat(String devicePath) throws TskCoreException;
 
 	private static native String getCurDirNat(long process);
-	
+
 	private static native boolean isImageSupportedNat(String imagePath);
 
+	private static native int finishImageWriterNat(long a_img_info);
+
+	private static native int getFinishImageProgressNat(long a_img_info);
+
+	private static native void cancelFinishImageNat(long a_img_info);
+
 }
