@@ -271,7 +271,7 @@ bool TskDbPostgreSQL::dbExists() {
 */
 int TskDbPostgreSQL::attempt_exec(const char *sql, const char *errfmt)
 {
-    if (!conn){
+    if (!conn) {
         tsk_error_reset();
         tsk_error_set_errno(TSK_ERR_AUTO_DB);
         tsk_error_set_errstr("Can't execute PostgreSQL query, not connected to database. Query: %s", sql);
@@ -279,6 +279,7 @@ int TskDbPostgreSQL::attempt_exec(const char *sql, const char *errfmt)
     }
 
     PGresult *res = PQexec(conn, sql); 
+
     if (!isQueryResultValid(res, sql)) {
         return 1;
     }
@@ -314,7 +315,7 @@ int TskDbPostgreSQL::isEscapedStringValid(const char *sql_str, const char *orig_
 
 /**
 * Execute SQL statement and returns PostgreSQL result sets in ASCII format. Sets TSK error values on error.
-* IMPORTANT: result set needs to be freed by caling PQclear(res) when no longer needed.
+* IMPORTANT: result set needs to be freed by calling PQclear(res) when no longer needed.
 * @returns Result set on success, NULL on error
 */
 PGresult* TskDbPostgreSQL::get_query_result_set(const char *sql, const char *errfmt)
@@ -345,7 +346,7 @@ PGresult* TskDbPostgreSQL::get_query_result_set(const char *sql, const char *err
 /**
 * Execute a statement and returns PostgreSQL result sets in binary format. Sets TSK error values on error.
 * IMPORTANT: PostgreSQL returns binary representations in network byte order, which need to be converted to the local byte order.
-* IMPORTANT: result set needs to be freed by caling PQclear(res) when no longer needed.
+* IMPORTANT: result set needs to be freed by calling PQclear(res) when no longer needed.
 * @returns Result set on success, NULL on error
 */
 PGresult* TskDbPostgreSQL::get_query_result_set_binary(const char *sql, const char *errfmt)
@@ -641,7 +642,9 @@ int TskDbPostgreSQL::createIndexes()
         attempt_exec("CREATE INDEX artifact_typeID ON blackboard_artifacts(artifact_type_id);",
         "Error creating artifact_objID index on blackboard_artifacts: %s\n") ||
         attempt_exec("CREATE INDEX attrsArtifactID ON blackboard_attributes(artifact_id);",
-        "Error creating artifact_id index on blackboard_attributes: %s\n") ;
+        "Error creating artifact_id index on blackboard_attributes: %s\n") ||
+		attempt_exec("CREATE INDEX mime_type ON tsk_files(dir_type,mime_type,type);",
+		"Error creating mime_type index on tsk_files: %s\n");
     /*attempt_exec("CREATE INDEX attribute_artifactTypeId ON blackboard_attributes(artifact_type_id);",
     "Error creating artifact_type_id index on blackboard_attributes: %s\n");
     */
@@ -1025,8 +1028,30 @@ int TskDbPostgreSQL::addFile(TSK_FS_FILE * fs_file, const TSK_FS_ATTR * fs_attr,
             return 1;
     }
 
-    char zSQL[2048];
-    snprintf(zSQL, 2048, "INSERT INTO tsk_files (fs_obj_id, obj_id, data_source_obj_id, type, attr_type, attr_id, name, meta_addr, meta_seq, dir_type, meta_type, dir_flags, meta_flags, size, crtime, ctime, atime, mtime, mode, gid, uid, md5, known, parent_path) "
+    char zSQL_fixed[2048];
+    zSQL_fixed[2047] = '\0';
+    char *zSQL_dynamic = NULL; // Only used if the query does not fit in the fixed length buffer
+    char *zSQL = zSQL_fixed;
+    int bufLen = 2048;
+
+    // Check if the path may be too long. The rest of the query should take up far less than 500 bytes.
+    if (strlen(name_sql) + strlen(escaped_path_sql) + 500 > bufLen) {
+        // The query may be long to fit in the standard buffer, so create a larger one.
+        // This should be a very rare case and allows us to not use malloc most of the time.
+        // The same buffer will be used for the slack file entry.
+        bufLen = strlen(escaped_path_sql) + strlen(name_sql) + 500;
+        if ((zSQL_dynamic = (char *)tsk_malloc(bufLen)) == NULL) {
+            free(name);
+            free(escaped_path);
+            PQfreemem(escaped_path_sql);
+            PQfreemem(name_sql);
+            return 1;
+        }
+        zSQL_dynamic[bufLen - 1] = '\0';
+        zSQL = zSQL_dynamic;
+    }
+
+    if (0 > snprintf(zSQL, bufLen - 1, "INSERT INTO tsk_files (fs_obj_id, obj_id, data_source_obj_id, type, attr_type, attr_id, name, meta_addr, meta_seq, dir_type, meta_type, dir_flags, meta_flags, size, crtime, ctime, atime, mtime, mode, gid, uid, md5, known, parent_path) "
         "VALUES ("
         "%" PRId64 ",%" PRId64 ","
         "%" PRId64 ","
@@ -1038,37 +1063,62 @@ int TskDbPostgreSQL::addFile(TSK_FS_FILE * fs_file, const TSK_FS_ATTR * fs_attr,
         "%llu,%llu,%llu,%llu,"
         "%d,%d,%d,%s,%d,"
         "%s)",
-        fsObjId, objId, 
+        fsObjId, objId,
         dataSourceObjId,
         TSK_DB_FILES_TYPE_FS,
         type, idx, name_sql,
-        fs_file->name->meta_addr, fs_file->name->meta_seq, 
+        fs_file->name->meta_addr, fs_file->name->meta_seq,
         fs_file->name->type, meta_type, fs_file->name->flags, meta_flags,
-        size, 
-        (unsigned long long)crtime, (unsigned long long)ctime,(unsigned long long) atime,(unsigned long long) mtime, 
+        size,
+        (unsigned long long)crtime, (unsigned long long)ctime, (unsigned long long) atime, (unsigned long long) mtime,
         meta_mode, gid, uid, NULL, known,
-        escaped_path_sql);
+        escaped_path_sql)) {
+
+            tsk_error_reset();
+            tsk_error_set_errno(TSK_ERR_AUTO_DB);
+            tsk_error_set_errstr("Error inserting file with object ID for: %" PRId64 , objId);
+            if (zSQL_dynamic != NULL) {
+                free(zSQL_dynamic);
+            }
+            free(name);
+            free(escaped_path);
+            PQfreemem(name_sql);
+            PQfreemem(escaped_path_sql);
+            return 1;
+    }
 
     if (attempt_exec(zSQL, "TskDbPostgreSQL::addFile: Error adding data to tsk_files table: %s\n")) {
         free(name);
         free(escaped_path);
         PQfreemem(name_sql);
         PQfreemem(escaped_path_sql);
+        if (zSQL_dynamic != NULL) {
+            free(zSQL_dynamic);
+        }
         return 1;
     }
 
+    //if dir, update parent id cache (do this before objId may be changed creating the slack file)
+    if (meta_type == TSK_FS_META_TYPE_DIR) {
+        std::string fullPath = std::string(path) + fs_file->name->name;
+        storeObjId(fsObjId, fs_file, fullPath.c_str(), objId);
+    }
+
     // Add entry for the slack space.
     // Current conditions for creating a slack file:
+    //   - File name is not empty, "." or ".."
     //   - Data is non-resident
-    //   - The allocated size is greater than the file size
+    //   - The allocated size is greater than the initialized file size
+    //     See github issue #756 on why initsize and not size. 
     //   - The data is not compressed
     if((fs_attr != NULL)
+           && ((strlen(name) > 0) && (!TSK_FS_ISDOT(name)))
            && (! (fs_file->meta->flags & TSK_FS_META_FLAG_COMP))
            && (fs_attr->flags & TSK_FS_ATTR_NONRES) 
-           && (fs_attr->nrd.allocsize >  fs_attr->size)){
+           && (fs_attr->nrd.allocsize >  fs_attr->nrd.initsize)){
         strncat(name, "-slack", 6);
         name_sql = PQescapeLiteral(conn, name, strlen(name));
-        TSK_OFF_T slackSize = fs_attr->nrd.allocsize - fs_attr->size;
+        TSK_OFF_T slackSize = fs_attr->nrd.allocsize - fs_attr->nrd.initsize;
 
         if (addObject(TSK_DB_OBJECT_TYPE_FILE, parObjId, objId)) {
             free(name);
@@ -1076,7 +1126,7 @@ int TskDbPostgreSQL::addFile(TSK_FS_FILE * fs_file, const TSK_FS_ATTR * fs_attr,
             return 1;
         }
 
-        snprintf(zSQL, 2048, "INSERT INTO tsk_files (fs_obj_id, obj_id, data_source_obj_id, type, attr_type, attr_id, name, meta_addr, meta_seq, dir_type, meta_type, dir_flags, meta_flags, size, crtime, ctime, atime, mtime, mode, gid, uid, md5, known, parent_path) "
+        if (0 > snprintf(zSQL, bufLen - 1, "INSERT INTO tsk_files (fs_obj_id, obj_id, data_source_obj_id, type, attr_type, attr_id, name, meta_addr, meta_seq, dir_type, meta_type, dir_flags, meta_flags, size, crtime, ctime, atime, mtime, mode, gid, uid, md5, known, parent_path) "
             "VALUES ("
             "%" PRId64 ",%" PRId64 ","
             "%" PRId64 ","
@@ -1088,31 +1138,44 @@ int TskDbPostgreSQL::addFile(TSK_FS_FILE * fs_file, const TSK_FS_ATTR * fs_attr,
             "%llu,%llu,%llu,%llu,"
             "%d,%d,%d,%s,%d,"
             "%s)",
-            fsObjId, objId, 
+            fsObjId, objId,
             dataSourceObjId,
             TSK_DB_FILES_TYPE_SLACK,
             type, idx, name_sql,
             fs_file->name->meta_addr, fs_file->name->meta_seq, 
-            fs_file->name->type, meta_type, fs_file->name->flags, meta_flags,
+            TSK_FS_NAME_TYPE_REG, TSK_FS_META_TYPE_REG, fs_file->name->flags, meta_flags,
             slackSize, 
             (unsigned long long)crtime, (unsigned long long)ctime,(unsigned long long) atime,(unsigned long long) mtime, 
             meta_mode, gid, uid, NULL, known,
-            escaped_path_sql);
+            escaped_path_sql)) {
+
+                tsk_error_reset();
+                tsk_error_set_errno(TSK_ERR_AUTO_DB);
+                tsk_error_set_errstr("Error inserting slack file with object ID for: %" PRId64, objId);
+                free(name);
+                free(escaped_path);
+                PQfreemem(name_sql);
+                PQfreemem(escaped_path_sql);
+                if (zSQL_dynamic != NULL) {
+                    free(zSQL_dynamic);
+                }
+                return 1;
+        }
 
         if (attempt_exec(zSQL, "TskDbPostgreSQL::addFile: Error adding data to tsk_files table: %s\n")) {
             free(name);
             free(escaped_path);
             PQfreemem(name_sql);
             PQfreemem(escaped_path_sql);
+            if (zSQL_dynamic != NULL) {
+                free(zSQL_dynamic);
+            }
             return 1;
         }
 
     }
-
-    //if dir, update parent id cache
-    if (meta_type == TSK_FS_META_TYPE_DIR) {
-        std::string fullPath = std::string(path) + fs_file->name->name;
-        storeObjId(fsObjId, fs_file, fullPath.c_str(), objId);
+    if (zSQL_dynamic != NULL) {
+        free(zSQL_dynamic);
     }
 
     // cleanup
@@ -1180,18 +1243,54 @@ int64_t TskDbPostgreSQL::findParObjId(const TSK_FS_FILE * fs_file, const char *p
 
     // Find the parent file id in the database using the parent metadata address
     // @@@ This should use sequence number when the new database supports it
-    char zSQL[1024];
+    char zSQL_fixed[1024];
+    zSQL_fixed[1023] = '\0';
+    char *zSQL_dynamic = NULL; // Only used if the query does not fit in the fixed length buffer
+    char *zSQL = zSQL_fixed;
+    int bufLen = 1024;
+
+    // Check if the path may be too long
+    if (strlen(escaped_parent_name_sql) + strlen(escaped_path_sql) + 200 > bufLen) {
+        // The parent path was too long to fit in the standard buffer, so create a larger one.
+        // This should be a very rare case and allows us to not use malloc most of the time.
+        bufLen = strlen(escaped_path_sql) + strlen(escaped_parent_name_sql) + 200;
+        if ((zSQL_dynamic = (char *)tsk_malloc(bufLen)) == NULL) {
+            PQfreemem(escaped_path_sql);
+            PQfreemem(escaped_parent_name_sql);
+            return -1;
+        }
+        zSQL_dynamic[bufLen - 1] = '\0';
+        zSQL = zSQL_dynamic;
+    }
+
     int expectedNumFileds = 1;
-    snprintf(zSQL, 1024, "SELECT obj_id FROM tsk_files WHERE meta_addr = %" PRIu64 " AND fs_obj_id = %" PRId64 " AND parent_path = %s AND name = %s", 
-        fs_file->name->par_addr, fsObjId, escaped_path_sql, escaped_parent_name_sql);
+    if (0 > snprintf(zSQL, bufLen - 1, "SELECT obj_id FROM tsk_files WHERE meta_addr = %" PRIu64 " AND fs_obj_id = %" PRId64 " AND parent_path = %s AND name = %s",
+        fs_file->name->par_addr, fsObjId, escaped_path_sql, escaped_parent_name_sql)) {
+
+            tsk_error_reset();
+            tsk_error_set_errno(TSK_ERR_AUTO_DB);
+            tsk_error_set_errstr("Error creating query for parent object ID for: %s", parentPath);
+            if (zSQL_dynamic != NULL) {
+                free(zSQL_dynamic);
+            }
+            PQfreemem(escaped_path_sql);
+            PQfreemem(escaped_parent_name_sql);
+            return -1;
+    }
     PGresult* res = get_query_result_set(zSQL, "TskDbPostgreSQL::findParObjId: Error selecting file id by meta_addr: %s (result code %d)\n");
 
     // check if a valid result set was returned
     if (verifyNonEmptyResultSetSize(zSQL, res, expectedNumFileds, "TskDbPostgreSQL::findParObjId: Unexpected number of columns in result set: Expected %d, Received %d\n")) {
+        if (zSQL_dynamic != NULL) {
+            free(zSQL_dynamic);
+        }
         return -1;
     }
 
     int64_t parObjId = atoll(PQgetvalue(res, 0, 0));
+    if (zSQL_dynamic != NULL) {
+        free(zSQL_dynamic);
+    }
     PQclear(res);
     PQfreemem(escaped_path_sql);
     PQfreemem(escaped_parent_name_sql);
@@ -1936,7 +2035,7 @@ int TskDbPostgreSQL::createSavepoint(const char *name)
     // In PostgreSQL savepoints can only be established when inside a transaction block.
     // NOTE: this will only work if we have 1 savepoint. If we use multiple savepoints, PostgreSQL will 
     // not allow us to call "BEGIN" inside a transaction. We will need to keep track of whether we are
-    // in transaction and only call "BEGIN" if we are not in trasaction. Alternatively we can keep
+    // in transaction and only call "BEGIN" if we are not in transaction. Alternatively we can keep
     // calling "BEGIN" every time we create a savepoint and simply ignore the error if there is one.
     // Also see note inside TskDbPostgreSQL::releaseSavepoint().
     snprintf(buff, 1024, "BEGIN;");
@@ -1987,7 +2086,7 @@ int TskDbPostgreSQL::releaseSavepoint(const char *name)
     // "COMMIT" when releasing the outer most savepoint.
     snprintf(buff, 1024, "COMMIT;");
 
-    return attempt_exec(buff, "Error commiting transaction: %s\n");
+    return attempt_exec(buff, "Error committing transaction: %s\n");
 }
 
 /** 
