@@ -462,7 +462,7 @@ tsk_img_open_external(
 /* This interface needs some more thought because the size of wchar is not standard.
  * If the goal i to provide a constant wchar interface, then we need to incorporate
  * UTF-32 to UTF-8 support as well.  If the goal is to provide a standard UTF-16
- * interface, we should use another type besiddes wchar_t.
+ * interface, we should use another type besides wchar_t.
  */
 TSK_IMG_INFO *
 tsk_img_open_utf16(int num_img,
@@ -478,7 +478,7 @@ tsk_img_open_utf16(int num_img,
         TSK_ENDIAN_ENUM endian;
         uint16_t tmp1;
 
-        /* The unicode conversio routines are primarily to convert Unicode
+        /* The unicode conversion routines are primarily to convert Unicode
          * in file and volume system images, which means they could be in
          * an endian ordering different from the local one.  We need to figure
          * out our local ordering so we can give it the right flag */
@@ -557,52 +557,3 @@ tsk_img_close(TSK_IMG_INFO * a_img_info)
     tsk_deinit_lock(&(a_img_info->cache_lock));
     a_img_info->close(a_img_info);
 }
-
-
-/**
- * \internal
- * Return the list of names for this open images. 
- * This is sort of a hack implementation and is internal only at this
- * point.  Returns pointers into the IMG_INFO structs and should not be
- * modified or freed.
- * @param a_img_info Image to pull names from
- * @param a_num_imgs Will contain number of elements in the return array.
- * @returns List of names.
- */
-const TSK_TCHAR **
-tsk_img_get_names(TSK_IMG_INFO *a_img_info, int *a_num_imgs) 
-{
-    if (a_img_info == NULL) {
-        tsk_error_reset();
-        tsk_error_set_errno(TSK_ERR_IMG_ARG);
-        tsk_error_set_errstr("tsk_img_get_names: IMG_INFO is NULL");
-        return NULL;
-    }
-    if (a_num_imgs == NULL) {
-        tsk_error_reset();
-        tsk_error_set_errno(TSK_ERR_IMG_ARG);
-        tsk_error_set_errstr("tsk_img_get_names: a_num_imgs is NULL");
-        return NULL;
-    }
-    *a_num_imgs = 0;
-
-    switch (a_img_info->itype) {
-        case TSK_IMG_TYPE_RAW:
-            {
-                IMG_RAW_INFO *raw_info = (IMG_RAW_INFO *)a_img_info;
-                *a_num_imgs = raw_info->num_img;
-                return raw_info->images;
-            }
-#if HAVE_LIBEWF
-        case TSK_IMG_TYPE_EWF_EWF:
-            {
-                IMG_EWF_INFO *ewf_info = (IMG_EWF_INFO *)a_img_info;
-                *a_num_imgs = ewf_info->num_imgs;
-                return ewf_info->images;
-            }
-            break;
-#endif
-        default:
-            return NULL;
-    }
-}
